# -*- coding: utf-8 -*-
"""File generated according to SciDataTool/Generator/ClassesRef/Output/Data.csv
WARNING! All changes made in this file will be lost!
"""
from os import linesep
from SciDataTool.Classes._check import check_init_dict, check_var, raise_
from SciDataTool.Functions.save import save
from SciDataTool.Classes._frozen import FrozenClass
from SciDataTool.Classes._check import InitUnKnowClassError


class Data(FrozenClass):
    """Abstract class for all kinds of data"""

    VERSION = 1
    # save method is available in all object
    save = save

    def __init__(self, symbol="", name="", unit="", symmetries={}, init_dict=None):
        """Constructor of the class. Can be use in two ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for Matrix, None will initialise the property with an empty Matrix
            for SciDataTool type, None will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary wiht every properties as keys
        ndarray or list can be given for Vector and Matrix
        object or dict can be given for SciDataTool Object"""
        if init_dict is not None:  # Initialisation by dict
            check_init_dict(init_dict, ["symbol", "name", "unit", "symmetries"])
            # Overwrite default value with init_dict content
            if "symbol" in list(init_dict.keys()):
                symbol = init_dict["symbol"]
            if "name" in list(init_dict.keys()):
                name = init_dict["name"]
            if "unit" in list(init_dict.keys()):
                unit = init_dict["unit"]
            if "symmetries" in list(init_dict.keys()):
                symmetries = init_dict["symmetries"]
        # Initialisation by argument
        self.parent = None
        self.symbol = symbol
        self.name = name
        self.unit = unit
        self.symmetries = symmetries
        # The class is frozen, for now it's impossible to add new properties
        self._freeze()

    def __str__(self):
        """Convert this objet in a readeable string (for print)"""
        Data_str = ""
        if self.parent is None:
            Data_str += "parent = None " + linesep
        else:
            Data_str += "parent = " + str(type(self.parent)) + " object" + linesep
        Data_str += 'symbol = "' + str(self.symbol) + '"' + linesep
        Data_str += 'name = "' + str(self.name) + '"' + linesep
        Data_str += 'unit = "' + str(self.unit) + '"' + linesep
        Data_str += "symmetries = " + str(self.symmetries)
        return Data_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""
        if type(other) != type(self):
            return False
        if other.symbol != self.symbol:
            return False
        if other.name != self.name:
            return False
        if other.unit != self.unit:
            return False
        if other.symmetries != self.symmetries:
            return False
        return True

    def as_dict(self):
        """Convert this objet in a json seriable dict (can be use in __init__)
        """
        Data_dict = dict()
        Data_dict["symbol"] = self.symbol
        Data_dict["name"] = self.name
        Data_dict["unit"] = self.unit
        Data_dict["symmetries"] = self.symmetries
        # The class name is added to the dict fordeserialisation purpose
        Data_dict["__class__"] = "Data"
        return Data_dict

    def _set_None(self):
        """Set all the properties to None (except SciDataTool object)"""
        self.symbol = None
        self.name = None
        self.unit = None
        self.symmetries = None

    def _get_symbol(self):
        """getter of symbol"""
        return self._symbol

    def _set_symbol(self, value):
        """setter of symbol"""
        check_var("symbol", value, "str")
        self._symbol = value

    # Symbol of the variable (in latex syntax)
    # Type : str
    symbol = property(
        fget=_get_symbol,
        fset=_set_symbol,
        doc=u"""Symbol of the variable (in latex syntax)""",
    )

    def _get_name(self):
        """getter of name"""
        return self._name

    def _set_name(self, value):
        """setter of name"""
        check_var("name", value, "str")
        self._name = value

    # Name of the physical quantity (to be used in plots)
    # Type : str
    name = property(
        fget=_get_name,
        fset=_set_name,
        doc=u"""Name of the physical quantity (to be used in plots)""",
    )

    def _get_unit(self):
        """getter of unit"""
        return self._unit

    def _set_unit(self, value):
        """setter of unit"""
        check_var("unit", value, "str")
        self._unit = value

    # Unit of the physical quantity (to be used in plots)
    # Type : str
    unit = property(
        fget=_get_unit,
        fset=_set_unit,
        doc=u"""Unit of the physical quantity (to be used in plots)""",
    )

    def _get_symmetries(self):
        """getter of symmetries"""
        return self._symmetries

    def _set_symmetries(self, value):
        """setter of symmetries"""
        check_var("symmetries", value, "dict")
        self._symmetries = value

    # List of the symmetries along each axis used for reducing storage
    # Type : dict
    symmetries = property(
        fget=_get_symmetries,
        fset=_set_symmetries,
        doc=u"""List of the symmetries along each axis used for reducing storage""",
    )



# -*- coding: utf-8 -*-
import logging
from os.path import normpath, join, abspath, dirname, isdir
from os import makedirs
from shutil import rmtree
from matplotlib import use

use("Qt5Agg")  # Use PyQt5 backend
TEST_DIR = abspath(dirname(__file__))
DATA_DIR = join(TEST_DIR, "Data")
LOG_DIR = join(TEST_DIR, "logtest.txt")
DOC_DIR = abspath(join(TEST_DIR, "..", "Doc"))
# Set logger for test
x = logging.getLogger("logtest")
x.setLevel(logging.DEBUG)
h = logging.FileHandler(LOG_DIR)
f = logging.Formatter("%(message)s")
h.setFormatter(f)
x.addHandler(h)
# Init the result folder for the test
save_path = join(TEST_DIR, "Results")
if isdir(save_path):  # Delete previous test result
    rmtree(save_path)
# To save all the plot geometry results
save_plot_path = join(save_path, "Plot")
makedirs(save_plot_path)
# To save the validation results
save_validation_path = join(save_path, "Validation")
makedirs(save_validation_path)
# To save the Save/Load .json results
save_load_path = join(save_path, "Save_Load")
makedirs(save_load_path)
# To save the GUI results
save_gui_path = join(save_path, "GUI")
makedirs(save_gui_path)
# To clean all the results at the end of the corresponding test
is_clean_result = False



from numpy import genfromtxt, squeeze, linspace, sin, abs as np_abs
from matplotlib.colors import ListedColormap
from scipy.io.wavfile import read
from unittest import TestCase
from os.path import join
import matplotlib.pyplot as plt
import unittest
from SciDataTool.Tests import save_validation_path as save_path
from SciDataTool.Classes.Simu1 import Simu1
from SciDataTool.Tests.Validation.Machine.SCIM_006 import SCIM_006
from SciDataTool.Classes.Output import Output
from SciDataTool.Classes.DataTime import DataTime
from SciDataTool.Classes.Data1D import Data1D
from SciDataTool.Classes.DataLinspace import DataLinspace
from SciDataTool.Classes.DataFreq import DataFreq
from SciDataTool.Tests import DATA_DIR
from SciDataTool.Classes.ImportMatlab import ImportMatlab

simu = Simu1(name="EM_SCIM_NL_006", machine=SCIM_006)
csv_file_Br = join(DATA_DIR, "default_proj_Br_time_angle.csv")
csv_file_time = join(DATA_DIR, "default_proj_time.csv")
csv_file_angle = join(DATA_DIR, "default_proj_angle.csv")
csv_file_aswl = join(DATA_DIR, "default_proj_LwiA.csv")
csv_file_freqs = join(DATA_DIR, "default_proj_freqs.csv")
csv_file_MTr_freqs = join(DATA_DIR, "default_proj_MTr_freqs.csv")
csv_file_MTr_wavenumber = join(DATA_DIR, "default_proj_MTr_wavenumber.csv")
mat_file_MTr = join(DATA_DIR, "default_proj_MTr.mat")
mat_file_Br_cfft2 = join(DATA_DIR, "default_proj_Br_cfft2.mat")
mat_file_Brfreqs = join(DATA_DIR, "default_proj_Brfreqs.mat")
mat_file_Brwavenumber = join(DATA_DIR, "default_proj_Brwavenumber.mat")
mat_file_colormap = join(DATA_DIR, "MANATEE_colormap.mat")
wav_file_sinus = join(DATA_DIR, "sinus_1000Hz_60dBSPL.wav")
wav_file_pinknoise = join(DATA_DIR, "PinkNoise_40dBpHz@1000Hz.wav")
wav_file_trafic = join(DATA_DIR, "trafic.wav")
# Read input files from Manatee
Br = genfromtxt(csv_file_Br, delimiter=",")
Br[0, 0] = -0.179266312
time = genfromtxt(csv_file_time, delimiter=",")
time[0] = 0
angle = genfromtxt(csv_file_angle, delimiter=",")
angle[0] = 0
aswl = genfromtxt(csv_file_aswl, delimiter=",")
aswl[0] = 0
freqs = genfromtxt(csv_file_freqs, delimiter=",")
freqs[0] = 0
MTr_freqs = genfromtxt(csv_file_MTr_freqs, delimiter=",")
MTr_freqs[0] = 0
MTr_wavenumber = genfromtxt(csv_file_MTr_wavenumber, delimiter=",")
MTr_wavenumber[0] = -8
MTr = squeeze(ImportMatlab(file_path=mat_file_MTr, var_name="XPwr").get_data())
Br_cfft2 = squeeze(ImportMatlab(file_path=mat_file_Br_cfft2, var_name="Fwr").get_data())
freqs_Br = squeeze(
    ImportMatlab(file_path=mat_file_Brfreqs, var_name="freqs").get_data()
)
wavenumber = squeeze(
    ImportMatlab(file_path=mat_file_Brwavenumber, var_name="orders").get_data()
)
newcolors = squeeze(
    ImportMatlab(file_path=mat_file_colormap, var_name="mymap").get_data()
)
colormap = ListedColormap(newcolors)
freq_max = 13000
r_max = 78
rate_sinus, sinus = read(wav_file_sinus)
if sinus.dtype == "int16":
    nb_bits = 16  # -> 16-bit wav files
elif sinus.dtype == "int32":
    nb_bits = 32  # -> 32-bit wav files
max_nb_bit = float(2 ** (nb_bits - 1))
sinus = sinus / (
    max_nb_bit
)  # samples is a numpy array of float representing the samples
rate_pinknoise, pinknoise = read(wav_file_pinknoise)
if pinknoise.dtype == "int16":
    nb_bits = 16  # -> 16-bit wav files
elif pinknoise.dtype == "int32":
    nb_bits = 32  # -> 32-bit wav files
max_nb_bit = float(2 ** (nb_bits - 1))
pinknoise = pinknoise / (
    max_nb_bit
)  # samples is a numpy array of float representing the samples
rate_trafic, trafic = read(wav_file_trafic)


class tests_dataobj(TestCase):
    # @unittest.skip
    def test_default_proj_Br_time_space(self):
        out = Output(simu=simu)
        out.post.legend_name = "Reference"
        # Build the data objects
        Time = Data1D(name="time", unit="s", symmetries={}, values=time)
        Angle = Data1D(name="angle", unit="rad", symmetries={}, values=angle)
        out.mag.Br = DataTime(
            symbol="B_r",
            name="Airgap radial flux density",
            unit="T",
            symmetries={},
            axes=[Time, Angle],
            normalizations={"space_order": 3},
            values=Br,
        )
        out2 = Output(simu=simu)
        out2.post.legend_name = "Periodicity 3"
        out2.post.line_color = "r--"
        # Reduce to 1/3 period
        Br_reduced = Br[0:672, 0:672]
        time_reduced = time[0:672]
        angle_reduced = angle[0:672]
        # Build the data objects
        Time = Data1D(
            name="time",
            unit="s",
            symmetries={"time": {"period": 3}},
            values=time_reduced,
        )
        Angle = Data1D(
            name="angle",
            unit="rad",
            symmetries={"angle": {"period": 3}},
            values=angle_reduced,
        )
        out2.mag.Br = DataTime(
            symbol="B_r",
            name="Airgap radial flux density",
            unit="T",
            symmetries={"time": {"period": 3}, "angle": {"period": 3}},
            axes=[Time, Angle],
            normalizations={},
            values=Br_reduced,
        )
        out3 = Output(simu=simu)
        out3.post.legend_name = "Linspace"
        out3.post.line_color = "r--"
        # Get linspace data
        t0 = time[0]
        tf = time[-1]
        deltat = time[1] - time[0]
        a0 = angle[0]
        deltaa = angle[1] - angle[0]
        Na = len(angle)
        # Build the data objects
        Time = DataLinspace(
            name="time",
            unit="s",
            symmetries={},
            initial=t0,
            final=tf,
            step=deltat,
            include_endpoint=True,
        )
        Angle = DataLinspace(
            name="angle",
            unit="rad",
            symmetries={},
            initial=a0,
            step=deltaa,
            number=Na,
            include_endpoint=False,
        )
        out3.mag.Br = DataTime(
            symbol="B_r",
            name="Airgap radial flux density",
            unit="T",
            symmetries={},
            axes=[Time, Angle],
            normalizations={"space_order": 3},
            values=Br,
        )
        out4 = Output(simu=simu)
        out4.post.legend_name = "Inverse FT"
        out4.post.line_color = "r--"
        # Build the data objects
        Freqs = Data1D(name="freqs", unit="Hz", symmetries={}, values=freqs_Br,)
        Wavenumber = Data1D(
            name="wavenumber", unit="", symmetries={}, values=wavenumber,
        )
        out4.mag.Br = DataFreq(
            symbol="B_r",
            name="Airgap radial flux density",
            unit="T",
            symmetries={},
            axes=[Freqs, Wavenumber],
            normalizations={},
            values=Br_cfft2,
        )
        # Plot the result by comparing the two simulation (sym / no sym)
        plt.close("all")
        out.plot_A_time("mag.Br", is_fft=True, freq_max=freq_max, out_list=[out2])
        fig = plt.gcf()
        fig.savefig(join(save_path, "test_default_proj_Br_dataobj_period.png"))
        # Plot the result by comparing the two simulation (Data1D / DataLinspace)
        plt.close("all")
        out.plot_A_space(
            "mag.Br", is_fft=True, is_spaceorder=True, r_max=r_max, out_list=[out3]
        )
        fig = plt.gcf()
        fig.savefig(join(save_path, "test_default_proj_Br_dataobj_linspace.png"))
        # Plot the result by comparing the two simulation (Data1D / DataLinspace)
        plt.close("all")
        out.plot_A_space("mag.Br", is_fft=True, r_max=r_max, out_list=[out4])
        fig = plt.gcf()
        fig.savefig(join(save_path, "test_default_proj_Br_dataobj_ift.png"))

    # @unittest.skip
    def test_default_proj_Br_cfft2(self):
        r_max = 78
        freq_max = 2500
        mag_max = 0.6
        N_stem = 100
        out = Output(simu=simu)
        # Build the data objects
        Time = Data1D(name="time", unit="s", symmetries={}, values=time)
        #        Angle = Data1D(name="angle", unit="rad", symmetries={"angle": {"period": 2}}, values=out.mag.angle[0:2048])
        Angle = Data1D(name="angle", unit="rad", symmetries={}, values=angle)
        out.mag.Br = DataTime(
            symbol="B_r",
            name="Airgap radial flux density",
            unit="T",
            symmetries={},
            axes=[Time, Angle],
            normalizations={},
            values=Br,
        )
        # Plot the result by comparing the two simulation (sym / no sym)
        plt.close("all")
        out.plot_A_cfft2(
            "mag.Br", freq_max=freq_max, r_max=r_max, mag_max=mag_max, N_stem=N_stem
        )
        fig = plt.gcf()
        fig.savefig(join(save_path, "test_default_proj_Br_dataobj_cfft2.png"))

    @unittest.skip
    def test_sinus_thirdoct(self):
        out = Output(simu=simu)
        # Build the data objects
        Time = DataLinspace(
            name="time",
            unit="s",
            symmetries={},
            initial=0,
            final=1.0,
            number=rate_sinus,
            include_endpoint=False,
        )
        SPL = DataTime(
            symbol="SPL",
            name="Sound Pressure Level",
            unit="Pa",
            symmetries={},
            axes=[Time],
            normalizations={"Pa": 2.0e-5},
            values=sinus[1:],
        )
        # Plot the result by comparing the two simulation (sym / no sym)
        plt.close("all")
        out.plot_ASWL(SPL)
        fig = plt.gcf()
        fig.savefig(join(save_path, "test_sinus_thirdoct_dataobj.png"))

    @unittest.skip
    def test_pinknoise_thirdoct(self):
        out = Output(simu=simu)
        # Build the data objects
        Time = DataLinspace(
            name="time",
            unit="s",
            symmetries={},
            initial=0,
            final=1.0,
            number=rate_pinknoise + 1,
            include_endpoint=False,
        )
        SPL = DataTime(
            symbol="SPL",
            name="Sound Pressure Level",
            unit="Pa",
            symmetries={},
            axes=[Time],
            normalizations={"Pa": 2.0e-5},
            values=pinknoise,
        )
        # Plot the result by comparing the two simulation (sym / no sym)
        plt.close("all")
        out.plot_ASWL(SPL)
        fig = plt.gcf()
        fig.savefig(join(save_path, "test_pinknoise_thirdoct_dataobj.png"))

    @unittest.skip
    def test_default_proj_aswl_thirdoct(self):
        out = Output(simu=simu)
        # Build the data objects
        Freqs = Data1D(name="freqs", unit="Hz", symmetries={}, values=freqs)
        ASWL = DataFreq(
            symbol="ASWL",
            name="A-weighted Sound Power Level",
            unit="dBA",
            symmetries={},
            axes=[Freqs],
            normalizations={},
            values=aswl,
        )
        # Plot the result by comparing the two simulation (sym / no sym)
        plt.close("all")
        out.plot_ASWL(ASWL, is_dBA=True)
        fig = plt.gcf()
        fig.savefig(join(save_path, "test_default_proj_ASWL_thirdoct_dataobj.png"))

    # @unittest.skip
    def test_default_proj_surf(self):
        out = Output(simu=simu)
        # Build the data objects
        Freqs = Data1D(name="freqs", unit="Hz", symmetries={}, values=freqs_Br,)
        Wavenumber = Data1D(
            name="wavenumber", unit="", symmetries={}, values=wavenumber,
        )
        out.mag.Br = DataFreq(
            symbol="B_r",
            name="Airgap radial flux density",
            unit="T",
            symmetries={},
            axes=[Freqs, Wavenumber],
            normalizations={},
            values=Br_cfft2,
        )
        # Plot the result by comparing the two simulation (sym / no sym)
        plt.close("all")
        out.plot_A_surf("mag.Br", t_max=0.06, colormap=colormap)
        fig = plt.gcf()
        fig.savefig(join(save_path, "test_default_proj_Br_surf_dataobj.png"))

    # @unittest.skip
    def test_default_proj_compare(self):
        out = Output(simu=simu)
        out.post.legend_name = "Br"
        # Build the data objects
        Time = Data1D(name="time", unit="s", symmetries={}, values=time)
        Angle = Data1D(name="angle", unit="rad", symmetries={}, values=angle)
        out.mag.Br = DataTime(
            symbol="B_r",
            name="Airgap radial flux density",
            unit="T",
            symmetries={},
            axes=[Time, Angle],
            normalizations={},
            values=Br,
        )
        out2 = Output(simu=simu)
        out2.post.legend_name = "0.2sin(375t-1.5)"
        out2.post.line_color = "r--"
        # Get linspace data
        t0 = 0.01
        tf = 0.04
        Nt = 3000
        time2 = linspace(0.01, 0.04, 3000, endpoint=True)
        # Compute sine function
        Br2 = 0.2 * sin(375 * time2 - 1.5)
        # Build the data objects
        Time = DataLinspace(
            name="time",
            unit="s",
            symmetries={},
            initial=t0,
            final=tf,
            number=Nt,
            include_endpoint=True,
        )
        out2.mag.Br = DataTime(
            symbol="B_r",
            name="Airgap radial flux density",
            unit="T",
            symmetries={},
            axes=[Time],
            normalizations={},
            values=Br2,
        )
        # Plot the result by comparing the two simulation (sym / no sym)
        plt.close("all")
        out.plot_A_time("mag.Br", out_list=[out2])
        fig = plt.gcf()
        fig.savefig(join(save_path, "test_default_proj_Br_compare.png"))

    # @unittest.skip
    def test_default_proj_fft2(self):
        out = Output(simu=simu)
        # Build the data objects
        Freqs = Data1D(name="freqs", unit="Hz", symmetries={}, values=MTr_freqs)
        Wavenumber = Data1D(
            name="wavenumber", unit="dimless", symmetries={}, values=MTr_wavenumber
        )
        out.mag.Br = DataFreq(
            symbol="MT_r",
            name="Radial stress applying on stator",
            unit="N/m2",
            symmetries={},
            axes=[Freqs, Wavenumber],
            normalizations={},
            values=MTr,
        )
        # Plot the result by comparing the two simulation (sym / no sym)
        plt.close("all")
        out.plot_A_fft2(
            "mag.Br", freq_max=13000, r_max=8, mag_max=50, colormap=colormap
        )
        fig = plt.gcf()
        fig.savefig(join(save_path, "test_default_proj_MTr_fft2_dataobj.png"))

    # @unittest.skip
    def test_default_proj_time_space(self):
        out = Output(simu=simu)
        # Build the data objects
        Time = Data1D(name="time", unit="s", symmetries={}, values=time)
        Angle = Data1D(name="angle", unit="rad", symmetries={}, values=angle)
        out.mag.Br = DataTime(
            symbol="B_r",
            name="Airgap radial flux density",
            unit="T",
            symmetries={},
            axes=[Time, Angle],
            normalizations={"space_order": 3},
            values=Br,
        )
        # Plot the result by comparing the two simulation (sym / no sym)
        plt.close("all")
        out.plot_A_time_space(
            "mag.Br", colormap=colormap, freq_max=freq_max, r_max=r_max
        )
        fig = plt.gcf()
        fig.savefig(join(save_path, "test_default_proj_Br_time_space_dataobj.png"))



# -*- coding: utf-8 -*-
"""File generated according to SciDataTool/Generator/ClassesRef/Output/Data1D.csv
WARNING! All changes made in this file will be lost!
"""
from os import linesep
from SciDataTool.Classes._check import set_array, check_init_dict, check_var, raise_
from SciDataTool.Functions.save import save
from SciDataTool.Classes.Data import Data

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from SciDataTool.Methods.Data1D.get_values import get_values
except ImportError as error:
    get_values = error
from numpy import array, array_equal
from SciDataTool.Classes._check import InitUnKnowClassError


class Data1D(Data):
    """Abstract class for all kinds of data"""

    VERSION = 1
    # cf Methods.Data1D.get_values
    if isinstance(get_values, ImportError):
        get_values = property(
            fget=lambda x: raise_(
                ImportError("Can't use Data1D method get_values: " + str(get_values))
            )
        )
    else:
        get_values = get_values
    # save method is available in all object
    save = save

    def __init__(
        self, values=[], symbol="", name="", unit="", symmetries={}, init_dict=None,
    ):
        """Constructor of the class. Can be use in two ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for Matrix, None will initialise the property with an empty Matrix
            for SciDataTool type, None will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary wiht every properties as keys
        ndarray or list can be given for Vector and Matrix
        object or dict can be given for SciDataTool Object"""
        if init_dict is not None:  # Initialisation by dict
            check_init_dict(
                init_dict, ["values", "symbol", "name", "unit", "symmetries"]
            )
            # Overwrite default value with init_dict content
            if "values" in list(init_dict.keys()):
                values = init_dict["values"]
            if "symbol" in list(init_dict.keys()):
                symbol = init_dict["symbol"]
            if "name" in list(init_dict.keys()):
                name = init_dict["name"]
            if "unit" in list(init_dict.keys()):
                unit = init_dict["unit"]
            if "symmetries" in list(init_dict.keys()):
                symmetries = init_dict["symmetries"]
        # Initialisation by argument
        # values can be None, a ndarray or a list
        set_array(self, "values", values)
        # Call Data init
        super(Data1D, self).__init__(
            symbol=symbol, name=name, unit=unit, symmetries=symmetries
        )
        # The class is frozen (in Data init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this objet in a readeable string (for print)"""
        Data1D_str = ""
        # Get the properties inherited from Data
        Data1D_str += super(Data1D, self).__str__() + linesep
        Data1D_str += "values = " + linesep + str(self.values)
        return Data1D_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""
        if type(other) != type(self):
            return False
        # Check the properties inherited from Data
        if not super(Data1D, self).__eq__(other):
            return False
        if not array_equal(other.values, self.values):
            return False
        return True

    def as_dict(self):
        """Convert this objet in a json seriable dict (can be use in __init__)
        """
        # Get the properties inherited from Data
        Data1D_dict = super(Data1D, self).as_dict()
        if self.values is None:
            Data1D_dict["values"] = None
        else:
            Data1D_dict["values"] = self.values.tolist()
        # The class name is added to the dict fordeserialisation purpose
        # Overwrite the mother class name
        Data1D_dict["__class__"] = "Data1D"
        return Data1D_dict

    def _set_None(self):
        """Set all the properties to None (except SciDataTool object)"""
        self.values = None
        # Set to None the properties inherited from Data
        super(Data1D, self)._set_None()

    def _get_values(self):
        """getter of values"""
        return self._values

    def _set_values(self, value):
        """setter of values"""
        if type(value) is list:
            try:
                value = array(value)
            except:
                pass
        check_var("values", value, "ndarray")
        self._values = value

    # ndarray of the field
    # Type : ndarray
    values = property(
        fget=_get_values, fset=_set_values, doc=u"""ndarray of the field"""
    )



# -*- coding: utf-8 -*-
"""File generated according to SciDataTool/Generator/ClassesRef/Output/DataFreq.csv
WARNING! All changes made in this file will be lost!
"""
from os import linesep
from SciDataTool.Classes._check import set_array, check_init_dict, check_var, raise_
from SciDataTool.Functions.save import save
from SciDataTool.Classes.DataND import DataND

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from SciDataTool.Methods.DataFreq.get_axis import get_axis
except ImportError as error:
    get_axis = error
try:
    from SciDataTool.Methods.DataFreq.get_FT_axis import get_FT_axis
except ImportError as error:
    get_FT_axis = error
try:
    from SciDataTool.Methods.DataFreq.get_along import get_along
except ImportError as error:
    get_along = error
try:
    from SciDataTool.Methods.DataFreq.get_FT_along import get_FT_along
except ImportError as error:
    get_FT_along = error
try:
    from SciDataTool.Methods.DataFreq.get_magnitude_along import get_magnitude_along
except ImportError as error:
    get_magnitude_along = error
try:
    from SciDataTool.Methods.DataFreq.get_phase_along import get_phase_along
except ImportError as error:
    get_phase_along = error
try:
    from SciDataTool.Methods.DataFreq.get_harmonics import get_harmonics
except ImportError as error:
    get_harmonics = error
try:
    from SciDataTool.Methods.DataFreq.get_nthoctave import get_nthoctave
except ImportError as error:
    get_nthoctave = error
try:
    from SciDataTool.Methods.DataFreq.find_periods import find_periods
except ImportError as error:
    find_periods = error
from numpy import array, array_equal
from SciDataTool.Classes._check import InitUnKnowClassError
from SciDataTool.Classes.Data import Data


class DataFreq(DataND):
    """Abstract class for all kinds of data"""

    VERSION = 1
    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.DataFreq.get_axis
    if isinstance(get_axis, ImportError):
        get_axis = property(
            fget=lambda x: raise_(
                ImportError("Can't use DataFreq method get_axis: " + str(get_axis))
            )
        )
    else:
        get_axis = get_axis
    # cf Methods.DataFreq.get_FT_axis
    if isinstance(get_FT_axis, ImportError):
        get_FT_axis = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use DataFreq method get_FT_axis: " + str(get_FT_axis)
                )
            )
        )
    else:
        get_FT_axis = get_FT_axis
    # cf Methods.DataFreq.get_along
    if isinstance(get_along, ImportError):
        get_along = property(
            fget=lambda x: raise_(
                ImportError("Can't use DataFreq method get_along: " + str(get_along))
            )
        )
    else:
        get_along = get_along
    # cf Methods.DataFreq.get_FT_along
    if isinstance(get_FT_along, ImportError):
        get_FT_along = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use DataFreq method get_FT_along: " + str(get_FT_along)
                )
            )
        )
    else:
        get_FT_along = get_FT_along
    # cf Methods.DataFreq.get_magnitude_along
    if isinstance(get_magnitude_along, ImportError):
        get_magnitude_along = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use DataFreq method get_magnitude_along: "
                    + str(get_magnitude_along)
                )
            )
        )
    else:
        get_magnitude_along = get_magnitude_along
    # cf Methods.DataFreq.get_phase_along
    if isinstance(get_phase_along, ImportError):
        get_phase_along = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use DataFreq method get_phase_along: " + str(get_phase_along)
                )
            )
        )
    else:
        get_phase_along = get_phase_along
    # cf Methods.DataFreq.get_harmonics
    if isinstance(get_harmonics, ImportError):
        get_harmonics = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use DataFreq method get_harmonics: " + str(get_harmonics)
                )
            )
        )
    else:
        get_harmonics = get_harmonics
    # cf Methods.DataFreq.get_nthoctave
    if isinstance(get_nthoctave, ImportError):
        get_nthoctave = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use DataFreq method get_nthoctave: " + str(get_nthoctave)
                )
            )
        )
    else:
        get_nthoctave = get_nthoctave
    # cf Methods.DataFreq.find_periods
    if isinstance(find_periods, ImportError):
        find_periods = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use DataFreq method find_periods: " + str(find_periods)
                )
            )
        )
    else:
        find_periods = find_periods
    # save method is available in all object
    save = save

    def __init__(
        self,
        axes=None,
        normalizations={},
        FTparameters={},
        values=None,
        symbol="",
        name="",
        unit="",
        symmetries={},
        init_dict=None,
    ):
        """Constructor of the class. Can be use in two ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for Matrix, None will initialise the property with an empty Matrix
            for SciDataTool type, None will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary wiht every properties as keys
        ndarray or list can be given for Vector and Matrix
        object or dict can be given for SciDataTool Object"""
        if init_dict is not None:  # Initialisation by dict
            check_init_dict(
                init_dict,
                [
                    "axes",
                    "normalizations",
                    "FTparameters",
                    "values",
                    "symbol",
                    "name",
                    "unit",
                    "symmetries",
                ],
            )
            # Overwrite default value with init_dict content
            if "axes" in list(init_dict.keys()):
                axes = init_dict["axes"]
            if "normalizations" in list(init_dict.keys()):
                normalizations = init_dict["normalizations"]
            if "FTparameters" in list(init_dict.keys()):
                FTparameters = init_dict["FTparameters"]
            if "values" in list(init_dict.keys()):
                values = init_dict["values"]
            if "symbol" in list(init_dict.keys()):
                symbol = init_dict["symbol"]
            if "name" in list(init_dict.keys()):
                name = init_dict["name"]
            if "unit" in list(init_dict.keys()):
                unit = init_dict["unit"]
            if "symmetries" in list(init_dict.keys()):
                symmetries = init_dict["symmetries"]
        # Initialisation by argument
        # Call DataND init
        super(DataFreq, self).__init__(
            axes=axes,
            normalizations=normalizations,
            FTparameters=FTparameters,
            values=values,
            symbol=symbol,
            name=name,
            unit=unit,
            symmetries=symmetries,
        )
        # The class is frozen (in DataND init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this objet in a readeable string (for print)"""
        DataFreq_str = ""
        # Get the properties inherited from DataND
        DataFreq_str += super(DataFreq, self).__str__() + linesep
        return DataFreq_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""
        if type(other) != type(self):
            return False
        # Check the properties inherited from DataND
        if not super(DataFreq, self).__eq__(other):
            return False
        return True

    def as_dict(self):
        """Convert this objet in a json seriable dict (can be use in __init__)
        """
        # Get the properties inherited from DataND
        DataFreq_dict = super(DataFreq, self).as_dict()
        # The class name is added to the dict fordeserialisation purpose
        # Overwrite the mother class name
        DataFreq_dict["__class__"] = "DataFreq"
        return DataFreq_dict

    def _set_None(self):
        """Set all the properties to None (except SciDataTool object)"""
        # Set to None the properties inherited from DataND
        super(DataFreq, self)._set_None()



# -*- coding: utf-8 -*-
from SciDataTool.Classes.Data import Data
from SciDataTool.Classes.Data1D import Data1D
from SciDataTool.Classes.DataFreq import DataFreq
from SciDataTool.Classes.DataLinspace import DataLinspace
from SciDataTool.Classes.DataND import DataND
from SciDataTool.Classes.DataTime import DataTime
import SciDataTool.Methods.Data1D
import SciDataTool.Methods.DataFreq
import SciDataTool.Methods.DataLinspace
import SciDataTool.Methods.DataND
import SciDataTool.Methods.DataTime
import SciDataTool.Functions


# -*- coding: utf-8 -*-
"""
Created on Mon Nov 17 10:04:21 2014
@author: pierre_b
"""
from numpy.random import random_sample, ranf

PYTHON_TYPE = ["float", "int", "str", "bool", "complex"]


def find_test_value(prop_dict, return_type):
    """Find a appropriate value to test the property of the return_type
    Parameters
    ----------
    prop_dict : dict
        Dictionary containing the property informations
    return_type : str
        type of the value for the test (can be different of the property one)
    Returns
    -------
    value: ?
        A "return_type" value for the test
    """
    assert return_type in PYTHON_TYPE or return_type == "ndarray"
    if return_type == "str":
        return "test with a string"
    if return_type == "bool":
        return True
    if return_type == "dict":
        return {"test": "Test for a dict", "test2": 5.2}
    if return_type == "int":
        if prop_dict["type"] in ["float", "int"]:
            return find_num_value(prop_dict, True)
        else:
            return 42
    if return_type == "complex":
        return random_sample() + 1j * random_sample()
    if return_type == "float":
        if prop_dict["type"] in ["float", "int"]:
            return find_num_value(prop_dict, False)
        else:
            return 2.0
    if return_type == "ndarray":
        return find_test_ndarray(prop_dict)


def find_test_ndarray(prop_dict):
    """Find a correct value to test ndarray
    Parameters
    ----------
    prop_dict : dict
        dictionary containing the information of the property to test
    Returns
    -------
    value: numpy.ndarray
        A value to test the property
    """
    assert prop_dict["type"] == "ndarray"
    var_min = prop_dict["min"]
    var_max = prop_dict["max"]
    # Adapt the value to the min/max
    if var_min == "" and var_max == "":  # No min and No max
        var_min = 0
        var_max = 100
    elif var_min == "" and var_max != "":  # No min but a max
        var_max = float(var_max)
        var_min = var_max - 100
    elif var_min != "" and var_max == "":  # No max but a min
        var_min = float(var_min)
        var_max = var_min + 100
    else:  # A min and a max
        var_min = float(var_min)
        var_max = float(var_max)
    # Random 4x4 float Matrix in [var_min, var_max]
    return ranf((4, 4)) * (var_max - var_min) + var_min


def find_num_value(prop_dict, is_int_return):
    """Find a value to test Double or Integer matching min/max
    Parameters
    ----------
    prop_dict : dict
        dictionary containing the information on the property to test
    is_int_return : bool
        To convert the value to int (if needed)
    Returns
    -------
    value : int/float
        value for the test
    """
    assert prop_dict["type"] in [
        "float",
        "int",
    ], "find_num_value is for numerical properties"
    assert type(is_int_return) is bool, "is_int_return must be a Boolean"
    var_min = prop_dict["min"]
    var_max = prop_dict["max"]
    # Adapt the value to the min/max
    if var_min == "" and var_max == "":  # No min and No max
        var_min = 0
        var_max = 100
    elif var_min == "" and var_max != "":  # No min but a max
        var_max = float(var_max)
        var_min = var_max - 100
    elif var_min != "" and var_max == "":  # No max but a min
        var_min = float(var_min)
        var_max = var_min + 100
    else:  # A min and a max
        var_min = float(var_min)
        var_max = float(var_max)
    # Random float in [var_min, var_max]
    test_value = random_sample() * (var_max - var_min) + var_min
    if is_int_return:  # We have to return an integer
        return int(test_value)
    else:
        return test_value


def is_type_list(type_name):
    """Check if the type_name is a list of pyleecan objects "[class_name]"
    Parameters
    ----------
    type_name : str
        name of the type to test
    Returns
    -------
    is_list : bool
        True if the type is a list of pyleecan objects
    """
    return type_name[0] == "[" and type_name[-1] == "]"


def is_type_dict(type_name):
    """Check if the type_name is a dict of pyleecan objects "{class_name}"
    Parameters
    ----------
    type_name : str
        name of the type to test
    Returns
    -------
    is_list : bool
        True if the type is a dict of pyleecan objects
    """
    return type_name[0] == "{" and type_name[-1] == "}"


class MissingTypeError(Exception):
    """ """

    pass



# -*- coding: utf-8 -*-
from numpy import pi, sqrt  # for eval
from SciDataTool.Functions import AxisError


def read_input_strings(args, axis_data):
    """Reads the string input into the "get_along" methods to define the axes
    Parameters
    ----------
    args: list
        list of string describing the requested axes
    axis_data: ndarray
        user-input values for the axes
    Returns
    -------
    list of axes data (axes_list)
    """
    axes_list = []
    for axis_str in args:
        axis_unit = "{SI}"
        # Detect unit
        if "{" in axis_str:
            elems = axis_str.split("{")
            axis_unit = "{" + elems[1]
            axis_str = elems[0]
        # Detect axis_data input
        if "axis_data" in axis_str:
            elems = axis_str.split("=axis_data")
            try:
                axes_list.append(
                    [
                        elems[0],
                        axis_unit,
                        "interval",
                        "values",
                        axis_data[int(elems[1]) - 1],
                    ]
                )
            except:
                try:
                    axes_list.append(
                        [elems[0], axis_unit, "interval", "values", axis_data[0]]
                    )
                except:
                    raise AxisError("ERROR: Absence of axis_data")
        # Detect interval
        elif "=[" in axis_str:
            elems = axis_str.split("=[")
            elems2 = elems[1].split(",")
            init_str = elems2[0]
            interval_init = eval(init_str)
            final_str = elems2[1].strip("]")
            interval_final = eval(final_str)
            axes_list.append(
                [
                    elems[0],
                    axis_unit,
                    "interval",
                    "values",
                    [interval_init, interval_final],
                ]
            )
        # Detect single value
        elif "=" in axis_str:
            elems = axis_str.split("=")
            axes_list.append(
                [elems[0], axis_unit, "single", "values", [eval(elems[1])]]
            )
        # Detect index input...
        elif "[" in axis_str:
            elems = axis_str.split("[")
            ind_str = elems[1].strip("]")
            # Range of indices
            if ":" in ind_str:
                elems2 = ind_str.split(":")
                indices = [i for i in range(int(elems2[0]), int(elems2[1]) + 1)]
                axes_list.append([elems[0], axis_unit, "interval", "indices", indices])
            # Single index
            else:
                indices = [int(ind_str)]
                axes_list.append([elems[0], axis_unit, "single", "indices", indices])
        # Whole axis
        else:
            axes_list.append([axis_str, axis_unit, "interval", "values", "whole"])
    return axes_list



# -*- coding: utf-8 -*-
from numpy import linspace, argmin, take, isclose, all, abs as np_abs
from scipy import interpolate


def get_common_base(values1, values2, is_extrap=False, is_downsample=False):
    """Returns a common base for vectors values1 and values2
    Parameters
    ----------
    values1: list
        values of the first axis
    values2: list
        values of the second axis
    is_extrap: bool
        Boolean indicating if we want to keep the widest vector and extrapolate the other one
    is_downsample: bool
        Boolean indicating if we want to keep the smallest number of points and downsample the other one
    Returns
    -------
    list of the common axis values
    """
    if is_extrap:
        initial = min(values1[0], values2[0])
        final = max(values1[-1], values2[-1])
    else:
        initial = max(values1[0], values2[0])
        final = min(values1[-1], values2[-1])
    if is_downsample:
        number = min(
            len([i for i in values1 if i >= initial and i <= final]),
            len([i for i in values2 if i >= initial and i <= final]),
        )
    else:
        number = max(
            len([i for i in values1 if i >= initial and i <= final]),
            len([i for i in values2 if i >= initial and i <= final]),
        )
    return linspace(initial, final, number, endpoint=True)


def get_interpolation(values, axis_values, new_axis_values):
    """Returns the interpolated field along one axis, given the new axis
    Parameters
    ----------
    values: ndarray
        1Darray of a field along one axis
    axis_values: list
        values of the original axis
    new_axis_values: list
        values of the new axis
    Returns
    -------
    ndarray of the interpolated field
    """
    if str(axis_values) == "whole":
        return values
    elif len(new_axis_values) == 1:
        idx = argmin(np_abs(axis_values - new_axis_values[0]))
        return take(values, [idx])
    elif len(axis_values) == len(new_axis_values) and all(
        isclose(axis_values, new_axis_values, rtol=1e-03)
    ):
        return values
    else:
        values_spline = interpolate.splrep(axis_values, values, s=0)
        return interpolate.splev(new_axis_values, values_spline, der=0)



# -*- coding: utf-8 -*-
"""File generated according to SciDataTool/Generator/ClassesRef/Output/DataLinspace.csv
WARNING! All changes made in this file will be lost!
"""
from os import linesep
from SciDataTool.Classes._check import check_init_dict, check_var, raise_
from SciDataTool.Functions.save import save
from SciDataTool.Classes.Data import Data

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from SciDataTool.Methods.DataLinspace.get_values import get_values
except ImportError as error:
    get_values = error
from SciDataTool.Classes._check import InitUnKnowClassError


class DataLinspace(Data):
    """Abstract class for all kinds of data"""

    VERSION = 1
    # cf Methods.DataLinspace.get_values
    if isinstance(get_values, ImportError):
        get_values = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use DataLinspace method get_values: " + str(get_values)
                )
            )
        )
    else:
        get_values = get_values
    # save method is available in all object
    save = save

    def __init__(
        self,
        initial=None,
        final=None,
        step=None,
        number=None,
        include_endpoint=False,
        symbol="",
        name="",
        unit="",
        symmetries={},
        init_dict=None,
    ):
        """Constructor of the class. Can be use in two ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for Matrix, None will initialise the property with an empty Matrix
            for SciDataTool type, None will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary wiht every properties as keys
        ndarray or list can be given for Vector and Matrix
        object or dict can be given for SciDataTool Object"""
        if init_dict is not None:  # Initialisation by dict
            check_init_dict(
                init_dict,
                [
                    "initial",
                    "final",
                    "step",
                    "number",
                    "include_endpoint",
                    "symbol",
                    "name",
                    "unit",
                    "symmetries",
                ],
            )
            # Overwrite default value with init_dict content
            if "initial" in list(init_dict.keys()):
                initial = init_dict["initial"]
            if "final" in list(init_dict.keys()):
                final = init_dict["final"]
            if "step" in list(init_dict.keys()):
                step = init_dict["step"]
            if "number" in list(init_dict.keys()):
                number = init_dict["number"]
            if "include_endpoint" in list(init_dict.keys()):
                include_endpoint = init_dict["include_endpoint"]
            if "symbol" in list(init_dict.keys()):
                symbol = init_dict["symbol"]
            if "name" in list(init_dict.keys()):
                name = init_dict["name"]
            if "unit" in list(init_dict.keys()):
                unit = init_dict["unit"]
            if "symmetries" in list(init_dict.keys()):
                symmetries = init_dict["symmetries"]
        # Initialisation by argument
        self.initial = initial
        self.final = final
        self.step = step
        self.number = number
        self.include_endpoint = include_endpoint
        # Call Data init
        super(DataLinspace, self).__init__(
            symbol=symbol, name=name, unit=unit, symmetries=symmetries
        )
        # The class is frozen (in Data init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this objet in a readeable string (for print)"""
        DataLinspace_str = ""
        # Get the properties inherited from Data
        DataLinspace_str += super(DataLinspace, self).__str__() + linesep
        DataLinspace_str += "initial = " + str(self.initial) + linesep
        DataLinspace_str += "final = " + str(self.final) + linesep
        DataLinspace_str += "step = " + str(self.step) + linesep
        DataLinspace_str += "number = " + str(self.number) + linesep
        DataLinspace_str += "include_endpoint = " + str(self.include_endpoint)
        return DataLinspace_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""
        if type(other) != type(self):
            return False
        # Check the properties inherited from Data
        if not super(DataLinspace, self).__eq__(other):
            return False
        if other.initial != self.initial:
            return False
        if other.final != self.final:
            return False
        if other.step != self.step:
            return False
        if other.number != self.number:
            return False
        if other.include_endpoint != self.include_endpoint:
            return False
        return True

    def as_dict(self):
        """Convert this objet in a json seriable dict (can be use in __init__)
        """
        # Get the properties inherited from Data
        DataLinspace_dict = super(DataLinspace, self).as_dict()
        DataLinspace_dict["initial"] = self.initial
        DataLinspace_dict["final"] = self.final
        DataLinspace_dict["step"] = self.step
        DataLinspace_dict["number"] = self.number
        DataLinspace_dict["include_endpoint"] = self.include_endpoint
        # The class name is added to the dict fordeserialisation purpose
        # Overwrite the mother class name
        DataLinspace_dict["__class__"] = "DataLinspace"
        return DataLinspace_dict

    def _set_None(self):
        """Set all the properties to None (except SciDataTool object)"""
        self.initial = None
        self.final = None
        self.step = None
        self.number = None
        self.include_endpoint = None
        # Set to None the properties inherited from Data
        super(DataLinspace, self)._set_None()

    def _get_initial(self):
        """getter of initial"""
        return self._initial

    def _set_initial(self, value):
        """setter of initial"""
        check_var("initial", value, "float")
        self._initial = value

    # First value
    # Type : float
    initial = property(fget=_get_initial, fset=_set_initial, doc=u"""First value""")

    def _get_final(self):
        """getter of final"""
        return self._final

    def _set_final(self, value):
        """setter of final"""
        check_var("final", value, "float")
        self._final = value

    # Last value
    # Type : float
    final = property(fget=_get_final, fset=_set_final, doc=u"""Last value""")

    def _get_step(self):
        """getter of step"""
        return self._step

    def _set_step(self, value):
        """setter of step"""
        check_var("step", value, "float")
        self._step = value

    # Step
    # Type : float
    step = property(fget=_get_step, fset=_set_step, doc=u"""Step""")

    def _get_number(self):
        """getter of number"""
        return self._number

    def _set_number(self, value):
        """setter of number"""
        check_var("number", value, "int")
        self._number = value

    # Number of steps
    # Type : int
    number = property(fget=_get_number, fset=_set_number, doc=u"""Number of steps""")

    def _get_include_endpoint(self):
        """getter of include_endpoint"""
        return self._include_endpoint

    def _set_include_endpoint(self, value):
        """setter of include_endpoint"""
        check_var("include_endpoint", value, "bool")
        self._include_endpoint = value

    # Boolean indicating if the endpoint must be included
    # Type : bool
    include_endpoint = property(
        fget=_get_include_endpoint,
        fset=_set_include_endpoint,
        doc=u"""Boolean indicating if the endpoint must be included""",
    )



# -*- coding: utf-8 -*-
"""File generated according to SciDataTool/Generator/ClassesRef/Output/DataND.csv
WARNING! All changes made in this file will be lost!
"""
from os import linesep
from SciDataTool.Classes._check import set_array, check_init_dict, check_var, raise_
from SciDataTool.Functions.save import save
from SciDataTool.Classes.Data import Data

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from SciDataTool.Methods.DataND.compare_along import compare_along
except ImportError as error:
    compare_along = error
try:
    from SciDataTool.Methods.DataND.compare_magnitude_along import (
        compare_magnitude_along,
    )
except ImportError as error:
    compare_magnitude_along = error
try:
    from SciDataTool.Methods.DataND.compare_phase_along import compare_phase_along
except ImportError as error:
    compare_phase_along = error
try:
    from SciDataTool.Methods.DataND.compress import compress
except ImportError as error:
    compress = error
try:
    from SciDataTool.Methods.DataND.set_Ftparameters import set_Ftparameters
except ImportError as error:
    set_Ftparameters = error
from numpy import array, array_equal
from SciDataTool.Classes._check import InitUnKnowClassError
from SciDataTool.Classes.Data import Data


class DataND(Data):
    """Abstract class for physical quantities depending on others"""

    VERSION = 1
    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.DataND.compare_along
    if isinstance(compare_along, ImportError):
        compare_along = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use DataND method compare_along: " + str(compare_along)
                )
            )
        )
    else:
        compare_along = compare_along
    # cf Methods.DataND.compare_magnitude_along
    if isinstance(compare_magnitude_along, ImportError):
        compare_magnitude_along = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use DataND method compare_magnitude_along: "
                    + str(compare_magnitude_along)
                )
            )
        )
    else:
        compare_magnitude_along = compare_magnitude_along
    # cf Methods.DataND.compare_phase_along
    if isinstance(compare_phase_along, ImportError):
        compare_phase_along = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use DataND method compare_phase_along: "
                    + str(compare_phase_along)
                )
            )
        )
    else:
        compare_phase_along = compare_phase_along
    # cf Methods.DataND.compress
    if isinstance(compress, ImportError):
        compress = property(
            fget=lambda x: raise_(
                ImportError("Can't use DataND method compress: " + str(compress))
            )
        )
    else:
        compress = compress
    # cf Methods.DataND.set_Ftparameters
    if isinstance(set_Ftparameters, ImportError):
        set_Ftparameters = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use DataND method set_Ftparameters: " + str(set_Ftparameters)
                )
            )
        )
    else:
        set_Ftparameters = set_Ftparameters
    # save method is available in all object
    save = save

    def __init__(
        self,
        axes=None,
        normalizations={},
        FTparameters={},
        values=None,
        symbol="",
        name="",
        unit="",
        symmetries={},
        init_dict=None,
    ):
        """Constructor of the class. Can be use in two ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for Matrix, None will initialise the property with an empty Matrix
            for SciDataTool type, None will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary wiht every properties as keys
        ndarray or list can be given for Vector and Matrix
        object or dict can be given for SciDataTool Object"""
        if init_dict is not None:  # Initialisation by dict
            check_init_dict(
                init_dict,
                [
                    "axes",
                    "normalizations",
                    "FTparameters",
                    "values",
                    "symbol",
                    "name",
                    "unit",
                    "symmetries",
                ],
            )
            # Overwrite default value with init_dict content
            if "axes" in list(init_dict.keys()):
                axes = init_dict["axes"]
            if "normalizations" in list(init_dict.keys()):
                normalizations = init_dict["normalizations"]
            if "FTparameters" in list(init_dict.keys()):
                FTparameters = init_dict["FTparameters"]
            if "values" in list(init_dict.keys()):
                values = init_dict["values"]
            if "symbol" in list(init_dict.keys()):
                symbol = init_dict["symbol"]
            if "name" in list(init_dict.keys()):
                name = init_dict["name"]
            if "unit" in list(init_dict.keys()):
                unit = init_dict["unit"]
            if "symmetries" in list(init_dict.keys()):
                symmetries = init_dict["symmetries"]
        # Initialisation by argument
        # axes can be None or a list of Data object
        self.axes = list()
        if type(axes) is list:
            for obj in axes:
                if obj is None:  # Default value
                    self.axes.append(Data())
                elif isinstance(obj, dict):
                    # Check that the type is correct (including daughter)
                    class_name = obj.get("__class__")
                    if class_name not in [
                        "Data",
                        "Data1D",
                        "DataFreq",
                        "DataLinspace",
                        "DataND",
                        "DataTime",
                    ]:
                        raise InitUnKnowClassError(
                            "Unknow class name " + class_name + " in init_dict for axes"
                        )
                    # Dynamic import to call the correct constructor
                    module = __import__(
                        "SciDataTool.Classes." + class_name, fromlist=[class_name]
                    )
                    class_obj = getattr(module, class_name)
                    self.axes.append(class_obj(init_dict=obj))
                else:
                    self.axes.append(obj)
        elif axes is None:
            self.axes = list()
        else:
            self.axes = axes
        self.normalizations = normalizations
        self.FTparameters = FTparameters
        # values can be None, a ndarray or a list
        set_array(self, "values", values)
        # Call Data init
        super(DataND, self).__init__(
            symbol=symbol, name=name, unit=unit, symmetries=symmetries
        )
        # The class is frozen (in Data init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this objet in a readeable string (for print)"""
        DataND_str = ""
        # Get the properties inherited from Data
        DataND_str += super(DataND, self).__str__() + linesep
        if len(self.axes) == 0:
            DataND_str += "axes = []"
        for ii in range(len(self.axes)):
            DataND_str += (
                "axes["
                + str(ii)
                + "] = "
                + str(self.axes[ii].as_dict())
                + "\n"
                + linesep
                + linesep
            )
        DataND_str += "normalizations = " + str(self.normalizations) + linesep
        DataND_str += "FTparameters = " + str(self.FTparameters) + linesep
        DataND_str += "values = " + linesep + str(self.values)
        return DataND_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""
        if type(other) != type(self):
            return False
        # Check the properties inherited from Data
        if not super(DataND, self).__eq__(other):
            return False
        if other.axes != self.axes:
            return False
        if other.normalizations != self.normalizations:
            return False
        if other.FTparameters != self.FTparameters:
            return False
        if not array_equal(other.values, self.values):
            return False
        return True

    def as_dict(self):
        """Convert this objet in a json seriable dict (can be use in __init__)
        """
        # Get the properties inherited from Data
        DataND_dict = super(DataND, self).as_dict()
        DataND_dict["axes"] = list()
        for obj in self.axes:
            DataND_dict["axes"].append(obj.as_dict())
        DataND_dict["normalizations"] = self.normalizations
        DataND_dict["FTparameters"] = self.FTparameters
        if self.values is None:
            DataND_dict["values"] = None
        else:
            DataND_dict["values"] = self.values.tolist()
        # The class name is added to the dict fordeserialisation purpose
        # Overwrite the mother class name
        DataND_dict["__class__"] = "DataND"
        return DataND_dict

    def _set_None(self):
        """Set all the properties to None (except SciDataTool object)"""
        for obj in self.axes:
            obj._set_None()
        self.normalizations = None
        self.FTparameters = None
        self.values = None
        # Set to None the properties inherited from Data
        super(DataND, self)._set_None()

    def _get_axes(self):
        """getter of axes"""
        for obj in self._axes:
            if obj is not None:
                obj.parent = self
        return self._axes

    def _set_axes(self, value):
        """setter of axes"""
        check_var("axes", value, "[Data]")
        self._axes = value
        for obj in self._axes:
            if obj is not None:
                obj.parent = self

    # List of the Data1D objects corresponding to the axes
    # Type : [Data]
    axes = property(
        fget=_get_axes,
        fset=_set_axes,
        doc=u"""List of the Data1D objects corresponding to the axes""",
    )

    def _get_normalizations(self):
        """getter of normalizations"""
        return self._normalizations

    def _set_normalizations(self, value):
        """setter of normalizations"""
        check_var("normalizations", value, "dict")
        self._normalizations = value

    # Normalizations available for the field and its axes
    # Type : dict
    normalizations = property(
        fget=_get_normalizations,
        fset=_set_normalizations,
        doc=u"""Normalizations available for the field and its axes""",
    )

    def _get_FTparameters(self):
        """getter of FTparameters"""
        return self._FTparameters

    def _set_FTparameters(self, value):
        """setter of FTparameters"""
        check_var("FTparameters", value, "dict")
        self._FTparameters = value

    # Tunable parameters for the Fourier Transforms
    # Type : dict
    FTparameters = property(
        fget=_get_FTparameters,
        fset=_set_FTparameters,
        doc=u"""Tunable parameters for the Fourier Transforms""",
    )

    def _get_values(self):
        """getter of values"""
        return self._values

    def _set_values(self, value):
        """setter of values"""
        if type(value) is list:
            try:
                value = array(value)
            except:
                pass
        check_var("values", value, "ndarray")
        self._values = value

    # Values of the field
    # Type : ndarray
    values = property(
        fget=_get_values, fset=_set_values, doc=u"""Values of the field"""
    )



# -*- coding: utf-8 -*-
class FTError(Exception):
    """Raised when there is an error within the Fourier Transform methods (Data Object)
    """

    pass


class AxisError(FTError):
    """Raised when the axis requested is not available (Data Object)
    """

    pass


class UnitError(FTError):
    """Raised when the unit requested is not available or does not match the existing one (Data Object)
    """

    pass


class NormError(FTError):
    """Raised when the unit requested is not available or does not match the existing one (Data Object)
    """

    pass
